@@ -5,6 +5,8 @@ operate. This is expected to be importable from any/all files within the
 subpackage and, thus, should not depend on them.
 """
 
+from __future__ import annotations
+
 import configparser
 import contextlib
 import locale
@@ -12,8 +14,9 @@ import logging
 import pathlib
 import re
 import sys
+from collections.abc import Iterator
 from itertools import chain, groupby, repeat
-from typing import TYPE_CHECKING, Dict, Iterator, List, Literal, Optional, Union
+from typing import TYPE_CHECKING, Literal
 
 from packaging.requirements import InvalidRequirement
 from packaging.version import InvalidVersion
@@ -27,7 +30,7 @@ if TYPE_CHECKING:
     from requests.models import Request, Response
 
     from pip._internal.metadata import BaseDistribution
-    from pip._internal.models.link import Link
+    from pip._internal.network.download import _FileDownload
     from pip._internal.req.req_install import InstallRequirement
 
 logger = logging.getLogger(__name__)
@@ -41,7 +44,7 @@ def _is_kebab_case(s: str) -> bool:
 
 
 def _prefix_with_indent(
-    s: Union[Text, str],
+    s: Text | str,
     console: Console,
     *,
     prefix: str,
@@ -77,13 +80,13 @@ class DiagnosticPipError(PipError):
     def __init__(
         self,
         *,
-        kind: 'Literal["error", "warning"]' = "error",
-        reference: Optional[str] = None,
-        message: Union[str, Text],
-        context: Optional[Union[str, Text]],
-        hint_stmt: Optional[Union[str, Text]],
-        note_stmt: Optional[Union[str, Text]] = None,
-        link: Optional[str] = None,
+        kind: Literal["error", "warning"] = "error",
+        reference: str | None = None,
+        message: str | Text,
+        context: str | Text | None,
+        hint_stmt: str | Text | None,
+        note_stmt: str | Text | None = None,
+        link: str | None = None,
     ) -> None:
         # Ensure a proper reference is provided.
         if reference is None:
@@ -232,7 +235,7 @@ class NoneMetadataError(PipError):
 
     def __init__(
         self,
-        dist: "BaseDistribution",
+        dist: BaseDistribution,
         metadata_name: str,
     ) -> None:
         """
@@ -293,8 +296,8 @@ class NetworkConnectionError(PipError):
     def __init__(
         self,
         error_msg: str,
-        response: Optional["Response"] = None,
-        request: Optional["Request"] = None,
+        response: Response | None = None,
+        request: Request | None = None,
     ) -> None:
         """
         Initialize NetworkConnectionError with  `request` and `response`
@@ -343,7 +346,7 @@ class MetadataInconsistent(InstallationError):
     """
 
     def __init__(
-        self, ireq: "InstallRequirement", field: str, f_val: str, m_val: str
+        self, ireq: InstallRequirement, field: str, f_val: str, m_val: str
     ) -> None:
         self.ireq = ireq
         self.field = field
@@ -360,7 +363,7 @@ class MetadataInconsistent(InstallationError):
 class MetadataInvalid(InstallationError):
     """Metadata is invalid."""
 
-    def __init__(self, ireq: "InstallRequirement", error: str) -> None:
+    def __init__(self, ireq: InstallRequirement, error: str) -> None:
         self.ireq = ireq
         self.error = error
 
@@ -378,7 +381,7 @@ class InstallationSubprocessError(DiagnosticPipError, InstallationError):
         *,
         command_description: str,
         exit_code: int,
-        output_lines: Optional[List[str]],
+        output_lines: list[str] | None,
     ) -> None:
         if output_lines is None:
             output_prompt = Text("See above for output.")
@@ -432,9 +435,9 @@ class HashErrors(InstallationError):
     """Multiple HashError instances rolled into one for reporting"""
 
     def __init__(self) -> None:
-        self.errors: List[HashError] = []
+        self.errors: list[HashError] = []
 
-    def append(self, error: "HashError") -> None:
+    def append(self, error: HashError) -> None:
         self.errors.append(error)
 
     def __str__(self) -> str:
@@ -468,7 +471,7 @@ class HashError(InstallationError):
 
     """
 
-    req: Optional["InstallRequirement"] = None
+    req: InstallRequirement | None = None
     head = ""
     order: int = -1
 
@@ -590,7 +593,7 @@ class HashMismatch(HashError):
         "someone may have tampered with them."
     )
 
-    def __init__(self, allowed: Dict[str, List[str]], gots: Dict[str, "_Hash"]) -> None:
+    def __init__(self, allowed: dict[str, list[str]], gots: dict[str, _Hash]) -> None:
         """
         :param allowed: A dict of algorithm names pointing to lists of allowed
             hex digests
@@ -615,12 +618,12 @@ class HashMismatch(HashError):
 
         """
 
-        def hash_then_or(hash_name: str) -> "chain[str]":
+        def hash_then_or(hash_name: str) -> chain[str]:
             # For now, all the decent hashes have 6-char names, so we can get
             # away with hard-coding space literals.
             return chain([hash_name], repeat("    or"))
 
-        lines: List[str] = []
+        lines: list[str] = []
         for hash_name, expecteds in self.allowed.items():
             prefix = hash_then_or(hash_name)
             lines.extend((f"        Expected {next(prefix)} {e}") for e in expecteds)
@@ -641,8 +644,8 @@ class ConfigurationFileCouldNotBeLoaded(ConfigurationError):
     def __init__(
         self,
         reason: str = "could not be loaded",
-        fname: Optional[str] = None,
-        error: Optional[configparser.Error] = None,
+        fname: str | None = None,
+        error: configparser.Error | None = None,
     ) -> None:
         super().__init__(error)
         self.reason = reason
@@ -677,7 +680,7 @@ class ExternallyManagedEnvironment(DiagnosticPipError):
 
     reference = "externally-managed-environment"
 
-    def __init__(self, error: Optional[str]) -> None:
+    def __init__(self, error: str | None) -> None:
         if error is None:
             context = Text(_DEFAULT_EXTERNALLY_MANAGED_ERROR)
         else:
@@ -704,7 +707,7 @@ class ExternallyManagedEnvironment(DiagnosticPipError):
         try:
             category = locale.LC_MESSAGES
         except AttributeError:
-            lang: Optional[str] = None
+            lang: str | None = None
         else:
             lang, _ = locale.getlocale(category)
         if lang is not None:
@@ -719,8 +722,8 @@ class ExternallyManagedEnvironment(DiagnosticPipError):
     @classmethod
     def from_config(
         cls,
-        config: Union[pathlib.Path, str],
-    ) -> "ExternallyManagedEnvironment":
+        config: pathlib.Path | str,
+    ) -> ExternallyManagedEnvironment:
         parser = configparser.ConfigParser(interpolation=None)
         try:
             parser.read(config, encoding="utf-8")
@@ -741,7 +744,7 @@ class ExternallyManagedEnvironment(DiagnosticPipError):
 class UninstallMissingRecord(DiagnosticPipError):
     reference = "uninstall-no-record-file"
 
-    def __init__(self, *, distribution: "BaseDistribution") -> None:
+    def __init__(self, *, distribution: BaseDistribution) -> None:
         installer = distribution.installer
         if not installer or installer == "pip":
             dep = f"{distribution.raw_name}=={distribution.version}"
@@ -768,7 +771,7 @@ class UninstallMissingRecord(DiagnosticPipError):
 class LegacyDistutilsInstall(DiagnosticPipError):
     reference = "uninstall-distutils-installed-package"
 
-    def __init__(self, *, distribution: "BaseDistribution") -> None:
+    def __init__(self, *, distribution: BaseDistribution) -> None:
         super().__init__(
             message=Text(f"Cannot uninstall {distribution}"),
             context=(
@@ -786,8 +789,8 @@ class InvalidInstalledPackage(DiagnosticPipError):
     def __init__(
         self,
         *,
-        dist: "BaseDistribution",
-        invalid_exc: Union[InvalidRequirement, InvalidVersion],
+        dist: BaseDistribution,
+        invalid_exc: InvalidRequirement | InvalidVersion,
     ) -> None:
         installed_location = dist.installed_location
 
@@ -816,17 +819,19 @@ class IncompleteDownloadError(DiagnosticPipError):
 
     reference = "incomplete-download"
 
-    def __init__(
-        self, link: "Link", received: int, expected: int, *, retries: int
-    ) -> None:
+    def __init__(self, download: _FileDownload) -> None:
         # Dodge circular import.
         from pip._internal.utils.misc import format_size
 
-        download_status = f"{format_size(received)}/{format_size(expected)}"
-        if retries:
-            retry_status = f"after {retries} attempts "
+        assert download.size is not None
+        download_status = (
+            f"{format_size(download.bytes_received)}/{format_size(download.size)}"
+        )
+        if download.reattempts:
+            retry_status = f"after {download.reattempts + 1} attempts "
             hint = "Use --resume-retries to configure resume attempt limit."
         else:
+            # Download retrying is not enabled.
             retry_status = ""
             hint = "Consider using --resume-retries to enable download resumption."
         message = Text(
@@ -836,7 +841,7 @@ class IncompleteDownloadError(DiagnosticPipError):
 
         super().__init__(
             message=message,
-            context=f"URL: {link.redacted_url}",
+            context=f"URL: {download.link.redacted_url}",
             hint_stmt=hint,
             note_stmt="This is an issue with network connectivity, not pip.",
         )
@@ -860,3 +865,17 @@ class ResolutionTooDeepError(DiagnosticPipError):
             ),
             link="https://pip.pypa.io/en/stable/topics/dependency-resolution/#handling-resolution-too-deep-errors",
         )
+
+
+class InstallWheelBuildError(DiagnosticPipError):
+    reference = "failed-wheel-build-for-install"
+
+    def __init__(self, failed: list[InstallRequirement]) -> None:
+        super().__init__(
+            message=(
+                "Failed to build installable wheels for some "
+                "pyproject.toml based projects"
+            ),
+            context=", ".join(r.name for r in failed),  # type: ignore
+            hint_stmt=None,
+        )
